/*
 File name : ArduionAlbum.ino
 author : fhn
 */

#define MAX_FILES 20
#define DISP_DELAY 500
#define ShowFileName 1  // 1 for yes  0 for no

#include <SPI.h>
#include <SD.h> 
#define TFT_RD   A0
#define TFT_WR   A1     
#define TFT_RS   A2        
#define TFT_CS   A3       
#define TFT_REST A4
#define TFT_XMAX 240
#define TFT_YMAX 320

#define COLOR_BLACK 0x000000
#define COLOR_GREY  0xE0E0E0
#define COLOR_RED   0xFF0000
#define COLOR_GREEN 0x00FF00
#define COLOR_BLUE  0x0000FF
#define COLOR_YELLOW 0xFFFF00
#define COLOR_CYAN  0x00FFFF
#define COLOR_MAGENTA 0xFF00FF
#define COLOR_WHITE 0xFFFFFF

#include "FONT_BASIC_16x8.H"

uint8_t FGC_R = 0;
uint8_t FGC_G = 0;
uint8_t FGC_B = 0;

uint8_t BGC_R = 0XFF;
uint8_t BGC_G = 0XFF;
uint8_t BGC_B = 0XFF;

char TMPSTR[12];
char filnams[MAX_FILES][14];

uint8_t fno=0;

void TFT_wrCom(uint8_t val)  
{   
 digitalWrite(TFT_RS,LOW);
 PORTD = (PORTD & B00000011) | ((val) & B11111100); 
 PORTB = (PORTB & B11111100) | ((val) & B00000011); 
 digitalWrite(TFT_WR,LOW);
 digitalWrite(TFT_WR,HIGH);
}

void TFT_wrDat(uint8_t val)
{
 digitalWrite(TFT_RS,HIGH);
 PORTD = (PORTD & B00000011) | ((val) & B11111100); 
 PORTB = (PORTB & B11111100) | ((val) & B00000011); 
 digitalWrite(TFT_WR,LOW);
 digitalWrite(TFT_WR,HIGH);
}

void TFT_setArea(uint16_t x1,uint16_t y1,uint16_t x2,uint16_t y2)
{
    TFT_wrCom(0x02A); 
    TFT_wrDat((x1>>8) & 0X00FF);
    TFT_wrDat(x1 & 0X00FF);
    TFT_wrDat((x2>>8) & 0X00FF);
    TFT_wrDat(x2 & 0X00FF);

    TFT_wrCom(0x02B); 
    TFT_wrDat((y1>>8) & 0X00FF);
    TFT_wrDat(y1 & 0X00FF);
    TFT_wrDat((y2>>8) & 0X00FF);
    TFT_wrDat(y2 & 0X00FF);
}

void TFT_init(void)
{
  for(int p=0;p<10;p++)
  {
    pinMode(p,OUTPUT);
  }
  pinMode(A0,OUTPUT);
  pinMode(A1,OUTPUT);
  pinMode(A2,OUTPUT);
  pinMode(A3,OUTPUT);
  pinMode(A4,OUTPUT);
  digitalWrite(A0, HIGH);
  digitalWrite(A1, HIGH);
  digitalWrite(A2, HIGH);
  digitalWrite(A3, HIGH);
  digitalWrite(A4, HIGH);
  delay(100);

  digitalWrite(TFT_REST,HIGH);
  delay(15); 
  digitalWrite(TFT_REST,LOW);
  delay(20);
  digitalWrite(TFT_REST,HIGH);
  delay(150);

  digitalWrite(TFT_CS,HIGH);
  digitalWrite(TFT_WR,HIGH);
  digitalWrite(TFT_CS,LOW);  //CS

  TFT_wrCom(0xCF); 
  TFT_wrDat(0x00);
  TFT_wrDat(0x81);
  TFT_wrDat(0x30);

  TFT_wrCom(0xED);  
  TFT_wrDat(0x64);
  TFT_wrDat(0x03);
  TFT_wrDat(0x12);
  TFT_wrDat(0x81);

  TFT_wrCom(0xE8); 
  TFT_wrDat(0x85);
  TFT_wrDat(0x10);
  TFT_wrDat(0x78);

  TFT_wrCom(0xCB);  
  TFT_wrDat(0x39);
  TFT_wrDat(0x2C);
  TFT_wrDat(0x00);
  TFT_wrDat(0x34);
  TFT_wrDat(0x02);

  TFT_wrCom(0xF7);  
  TFT_wrDat(0x20);

  TFT_wrCom(0xEA);  
  TFT_wrDat(0x00);
  TFT_wrDat(0x00);

  TFT_wrCom(0xB1);  
  TFT_wrDat(0x00);
  TFT_wrDat(0x18);

  TFT_wrCom(0xB6);    
  TFT_wrDat(0x0A);
  TFT_wrDat(0xA2);

  TFT_wrCom(0xC0);    
  TFT_wrDat(0x21);
  
  TFT_wrCom(0xC1);    
  TFT_wrDat(0x11);   

  TFT_wrCom(0xC5);    
  TFT_wrDat(0x3F);
  TFT_wrDat(0x3C);

  TFT_wrCom(0xC7);  
  TFT_wrDat(0xB5);

  TFT_wrCom(0x36);  
  TFT_wrDat(0x48);

  TFT_wrCom(0xF2);  
  TFT_wrDat(0x00);

  TFT_wrCom(0x26);  
  TFT_wrDat(0x01);

  TFT_wrCom(0xE0);    
  TFT_wrDat(0x0F);
  TFT_wrDat(0x26);
  TFT_wrDat(0x24);
  TFT_wrDat(0x0B);
  TFT_wrDat(0x0E);
  TFT_wrDat(0x09);
  TFT_wrDat(0x54);
  TFT_wrDat(0xA8);
  TFT_wrDat(0x46);
  TFT_wrDat(0x0C);
  TFT_wrDat(0x17);
  TFT_wrDat(0x09);
  TFT_wrDat(0x0F);
  TFT_wrDat(0x07);
  TFT_wrDat(0x00);

  TFT_wrCom(0xE1);    
  TFT_wrDat(0x00);
  TFT_wrDat(0x19);
  TFT_wrDat(0x1B);
  TFT_wrDat(0x04);
  TFT_wrDat(0x10);
  TFT_wrDat(0x07);
  TFT_wrDat(0x2A);
  TFT_wrDat(0x47);
  TFT_wrDat(0x39);
  TFT_wrDat(0x03);
  TFT_wrDat(0x06);
  TFT_wrDat(0x06);
  TFT_wrDat(0x30);
  TFT_wrDat(0x38);
  TFT_wrDat(0x0F);

  TFT_wrCom(0x11);  
  delay(150);
  TFT_wrCom(0x29);  
}

void TFT_drawPix ( uint16_t xp,uint16_t yp )
{
    uint16_t x = TFT_XMAX-xp;
    uint16_t y = yp;
    if (x<0 || y<0 || x>239 || y>319 )
      return;
    digitalWrite(TFT_CS,LOW);
    TFT_wrCom(0x02A); // column address from and to
    TFT_wrDat((x>>8) & 0X00FF);
    TFT_wrDat(x & 0X00FF);
    TFT_wrDat((x>>8) & 0X00FF);
    TFT_wrDat(x & 0X00FF);

    TFT_wrCom(0x02B); // page address
    TFT_wrDat((y>>8) & 0X00FF);
    TFT_wrDat(y & 0X00FF);
    TFT_wrDat((y>>8) & 0X00FF);
    TFT_wrDat(y & 0X00FF);

    TFT_wrCom(0x02c); //write_memory_start
    digitalWrite(TFT_RS,HIGH);
    TFT_wrDat(FGC_R);
    TFT_wrDat(FGC_G);
    TFT_wrDat(FGC_B);
    digitalWrite(TFT_CS,HIGH); 
}

void TFT_cls( )                   
{  
  uint16_t x,y;
  digitalWrite(TFT_CS,LOW);
  TFT_setArea(0,0,TFT_XMAX,TFT_YMAX);
  TFT_wrCom(0x02c); 
  digitalWrite(TFT_RS,HIGH);
  
  for(x=0;x<TFT_XMAX;x++)
  {
    for(y=0;y<TFT_YMAX;y++)
    {
      TFT_wrDat(BGC_R);
      TFT_wrDat(BGC_G);
      TFT_wrDat(BGC_B);
    }
  }
  digitalWrite(TFT_CS,HIGH);   
}

void TFT_drawTxtLin ( uint16_t cod , int p_x, int p_y)
{
  int x=p_x, y=p_y;
  for ( int i=0; i<16; i++ )
  {
    if ( cod & 0x8000 )
      TFT_drawPix ( x, y ); 
    y=y+1;
    cod = cod<<1;
  }
}

void TFT_drawStr ( char *str   , uint16_t x, uint16_t y)
{
  int px=x, py=y;
  {
    for ( uint8_t i=0; str[i]!=0; i++ ) 
    {
      uint8_t chr = str[i];
      uint16_t ch = (chr-' ')*8*2;
      for ( uint8_t c=0; c<8; c++ )
      {
        uint8_t byt1 = (pgm_read_byte(&FONT_BASIC_16_8[ch+(0)+c]));
        uint8_t byt2 = (pgm_read_byte(&FONT_BASIC_16_8[ch+(8)+c]));
        uint16_t cod = (byt2<<8) | byt1;
        TFT_drawTxtLin ( cod , px, py);
        px=px+1;
      }
      if ( chr=='M' || chr=='W' )
      {
          TFT_drawTxtLin (0x2002, px, py);
          px=px+1;
      }
      px=px+2;
    }
  }
}




void SD_init() 
{
 
  TFT_drawStr ( "SD CHECK", 1, 300);

  if (!SD.begin(10)) 
  {
    TFT_drawStr ( "ERR", 150, 300);
    return;
  }
  TFT_drawStr ( "OK", 150, 300);
}


void SD_lstfils(File dir)
{
  while (true) 
  {
    File fileitem =  dir.openNextFile();
    if (! fileitem) 
    {
      TFT_drawStr ( "ALL FILES LISTED", 1, 200);
      break;
    }
    
    if (fileitem.isDirectory()) {
      //nothing to with folder
    } else {
      if (fno>=MAX_FILES)
      {
        fileitem.close();
        TFT_drawStr ( "FILES MORE THAN MAX_FILES", 1, 220);
        break;
      }
      strcpy ( filnams[fno], fileitem.name());
      filnams[fno++][13]=0;
    }
    fileitem.close();
  }
}



int SD_rdFil ( char* filnam )
{
  int i, dat;
  char isFileValid = 'N';
  uint16_t datastart=0;
  uint16_t colorbits =0;
  unsigned long cols=0;
  unsigned long rows=0;
  uint16_t compression=0;

  if (ShowFileName)
    TFT_drawStr ( filnam, 1, 10);
  File myFile = SD.open(filnam, FILE_READ);
  isFileValid='N';
  
  if (myFile) 
  {
    int i=0;
    if (myFile.read()!='B' || myFile.read()!='M')
    {
      myFile.close();
      return -2;
    }
    for ( i=0; i<8; i++)
      myFile.read();
    datastart = myFile.read();
    datastart = datastart + myFile.read()*256;
    for ( i=0; i<6; i++)
      myFile.read();
    cols = myFile.read();
    cols = cols + myFile.read()*256;
    myFile.read();
    myFile.read();
    rows = myFile.read();
    rows = rows + myFile.read()*256;
    for ( i=0; i<4; i++)
      myFile.read();
    colorbits = myFile.read();
    myFile.read();
    compression = myFile.read();
 
    if ( colorbits==24 && rows==320 && cols==240 && compression==0 )
      isFileValid='Y';
    else
    {
      myFile.close();
      return -5;
    }
    
      for ( i=31; i<datastart; i++)
        myFile.read();
    
      for (int y=0; myFile.available() && y<320; y++) 
      {
        for (int x=0; myFile.available() && x<240; x++) 
        {
          FGC_B = myFile.read();
          FGC_G = myFile.read();
          FGC_R = myFile.read();
          
          TFT_drawPix(x,y);
        }
      }
    
    myFile.close();
    delay(DISP_DELAY);
  }
}


void setup()
{
  delay (500);
  TFT_init();
  TFT_cls( );
  TFT_drawStr ( "WELCOME", 75, 180);
  TFT_drawStr ( "TO", 100, 140);
  TFT_drawStr ( "ARDUINO-ALBUM", 50, 100);
  delay (500);
  TFT_cls( );

  SD_init();
  File root = SD.open("/");
  SD_lstfils(root);
  root.close();
  TFT_cls( );
  TFT_drawStr ( "TOTAL FILES:", 1, 300);
  TFT_drawStr ( itoa(fno, TMPSTR, 10), 130, 300);
  for ( int i=0; i<fno; i++ )
    TFT_drawStr ( filnams[i], 1, 280-(i*16));
  delay(500);
}

void loop() 
{
  for ( uint8_t i=0; i<fno; i++ )
  {
    SD_rdFil ( filnams[i] );
  }
  TFT_cls( );
}
